# GitHub System

## Table of Contents 

  - [Getting Started](#getting-started)
    - [Helpful Background Information](#helpful-background-information)
    - [Prerequisites](#prerequisites)
    - [How to Install](#how-to-install)
    - [Testing](#testing)
  - [Configuration](#configuration)
    - [Example Properties](#example-properties)
    - [Connection Properties](#connection-properties)
    - [Authentication Properties](#authentication-properties)
    - [Healthcheck Properties](#healthcheck-properties)
    - [Request Properties](#request-properties)
    - [SSL Properties](#ssl-properties)
    - [Throttle Properties](#throttle-properties)
    - [Proxy Properties](#proxy-properties)
    - [Mongo Properties](#mongo-properties)
    - [Device Broker Properties](#device-broker-properties)
  - [Using this Adapter](#using-this-adapter)
    - [Generic Adapter Calls](#generic-adapter-calls)
    - [Adapter Cache Calls](#adapter-cache-calls)
    - [Adapter Broker Calls](#adapter-broker-calls)
    - [Specific Adapter Calls](#specific-adapter-calls)
    - [Authentication](#authentication)
  - [Additional Information](#additional-information)
    - [Enhancements](#enhancements)
    - [Contributing](#contributing)
    - [Helpful Links](#helpful-links)
    - [Node Scripts](#node-scripts)
  - [Troubleshoot](#troubleshoot)
    - [Connectivity Issues](#connectivity-issues)
    - [Functional Issues](#functional-issues)

## Getting Started

These instructions will help you get a copy of the project on your local machine for development and testing. Reading this section is also helpful for deployments as it provides you with pertinent information on prerequisites and properties.

### Helpful Background Information

There is <a href="https://docs.itential.com/opensource/docs/adapters" target="_blank">Adapter documentation available on the Itential Documentation Site</a>. This documentation includes information and examples that are helpful for:

```text
Authentication
IAP Service Instance Configuration
Code Files
Endpoint Configuration (Action & Schema)
Mock Data
Adapter Generic Methods
Headers
Security
Linting and Testing
Build an Adapter
Troubleshooting an Adapter
```

Others will be added over time.
Want to build a new adapter? Use the <a href="https://adapters.itential.io" target="_blank">Itential Adapter Builder</a>

### Prerequisites

The following is a list of required packages for installation on the system the adapter will run on:

```text
Node.js
npm
Git
```

The following list of packages are required for Itential opensource adapters or custom adapters that have been built utilizing the Itential Adapter Builder. You can install these packages by running npm install inside the adapter directory.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Package</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
  </tr>
  <tr>
    <td style="padding:15px">@itentialopensource/adapter-utils</td>
    <td style="padding:15px">Runtime library classes for all adapters;  includes request handling, connection, authentication throttling, and translation.</td>
  </tr>
  <tr>
    <td style="padding:15px">ajv</td>
    <td style="padding:15px">Required for validation of adapter properties to integrate with GitHub.</td>
  </tr>
  <tr>
    <td style="padding:15px">axios</td>
    <td style="padding:15px">Utilized by the node scripts that are included with the adapter; helps to build and extend the functionality.</td>
  </tr>
  <tr>
    <td style="padding:15px">commander</td>
    <td style="padding:15px">Utilized by the node scripts that are included with the adapter; helps to build and extend the functionality.</td>
  </tr>
  <tr>
    <td style="padding:15px">dns-lookup-promise</td>
    <td style="padding:15px">Utilized by the node scripts that are included with the adapter; helps to build and extend the functionality.</td>
  </tr>
  <tr>
    <td style="padding:15px">fs-extra</td>
    <td style="padding:15px">Utilized by the node scripts that are included with the adapter; helps to build and extend the functionality.</td>
  </tr>
  <tr>
    <td style="padding:15px">mocha</td>
    <td style="padding:15px">Testing library that is utilized by some of the node scripts that are included with the adapter.</td>
  </tr>
  <tr>
    <td style="padding:15px">mocha-param</td>
    <td style="padding:15px">Testing library that is utilized by some of the node scripts that are included with the adapter.</td>
  </tr>
  <tr>
    <td style="padding:15px">mongodb</td>
    <td style="padding:15px">Utilized by the node scripts that are included with the adapter; helps to build and extend the functionality.</td>
  </tr>
  <tr>
    <td style="padding:15px">nyc</td>
    <td style="padding:15px">Testing coverage library that is utilized by some of the node scripts that are included with the adapter.</td>
  </tr>
  <tr>
    <td style="padding:15px">ping</td>
    <td style="padding:15px">Utilized by the node scripts that are included with the adapter; helps to build and extend the functionality.</td>
  </tr>
  <tr>
    <td style="padding:15px">readline-sync</td>
    <td style="padding:15px">Utilized by the node script that comes with the adapter;  helps to test unit and integration functionality.</td>
  </tr>
  <tr>
    <td style="padding:15px">semver</td>
    <td style="padding:15px">Utilized by the node scripts that are included with the adapter; helps to build and extend the functionality.</td>
  </tr>
  <tr>
    <td style="padding:15px">winston</td>
    <td style="padding:15px">Utilized by the node scripts that are included with the adapter; helps to build and extend the functionality.</td>
  </tr>
</table>
<br>

If you are developing and testing a custom adapter, or have testing capabilities on an Itential opensource adapter, you will need to install these packages as well.

```text
chai
eslint
eslint-config-airbnb-base
eslint-plugin-import
eslint-plugin-json
testdouble
```

### How to Install

1. Set up the name space location in your IAP node_modules.

```bash
cd /opt/pronghorn/current/node_modules (* could be in a different place)
if the @itentialopensource directory does not exist, create it:
    mkdir @itentialopensource
```

2. Clone/unzip/tar the adapter into your IAP environment.

```bash
cd \@itentialopensource
git clone git@gitlab.com:\@itentialopensource/adapters/adapter-github
or
unzip adapter-github.zip
or
tar -xvf adapter-github.tar
```

3. Run the adapter install script.

```bash
cd adapter-github
npm install
npm run lint:errors
npm run test
```

4. Restart IAP

```bash
systemctl restart pronghorn
```

5. Create an adapter service instance configuration in IAP Admin Essentials GUI

6. Copy the properties from the sampleProperties.json and paste them into the service instance configuration in the inner/second properties field.

7. Change the adapter service instance configuration (host, port, credentials, etc) in IAP Admin Essentials GUI


For an easier install of the adapter use npm run adapter:install, it will install the adapter in IAP. Please note that it can be dependent on where the adapter is installed and on the version of IAP so it is subject to fail. If using this, you can replace step 3-5 above with these:

3. Install adapter dependencies and check the adapter.

```bash
cd adapter-github
npm run adapter:install
```

4. Restart IAP

```bash
systemctl restart pronghorn
```

5. Change the adapter service instance configuration (host, port, credentials, etc) in IAP Admin Essentials GUI


### Testing

Mocha is generally used to test all Itential Opensource Adapters. There are unit tests as well as integration tests performed. Integration tests can generally be run as standalone using mock data and running the adapter in stub mode, or as integrated. When running integrated, every effort is made to prevent environmental failures, however there is still a possibility.

#### Unit Testing

Unit Testing includes testing basic adapter functionality as well as error conditions that are triggered in the adapter prior to any integration. There are two ways to run unit tests. The prefered method is to use the testRunner script; however, both methods are provided here.

```bash
node utils/testRunner --unit

npm run test:unit
npm run test:baseunit
```

To add new unit tests, edit the `test/unit/adapterTestUnit.js` file. The tests that are already in this file should provide guidance for adding additional tests.

#### Integration Testing - Standalone

Standalone Integration Testing requires mock data to be provided with the entities. If this data is not provided, standalone integration testing will fail. When the adapter is set to run in stub mode (setting the stub property to true), the adapter will run through its code up to the point of making the request. It will then retrieve the mock data and return that as if it had received that data as the response from GitHub. It will then translate the data so that the adapter can return the expected response to the rest of the Itential software. Standalone is the default integration test.

Similar to unit testing, there are two ways to run integration tests. Using the testRunner script is better because it prevents you from having to edit the test script; it will also resets information after testing is complete so that credentials are not saved in the file.

```bash
node utils/testRunner
  answer no at the first prompt

npm run test:integration
```

To add new integration tests, edit the `test/integration/adapterTestIntegration.js` file. The tests that are already in this file should provide guidance for adding additional tests.

#### Integration Testing

Integration Testing requires connectivity to GitHub. By using the testRunner script it prevents you from having to edit the integration test. It also resets the integration test after the test is complete so that credentials are not saved in the file.

> **Note**: These tests have been written as a best effort to make them work in most environments. However, the Adapter Builder often does not have the necessary information that is required to set up valid integration tests. For example, the order of the requests can be very important and data is often required for `creates` and `updates`. Hence, integration tests may have to be enhanced before they will work (integrate) with GitHub. Even after tests have been set up properly, it is possible there are environmental constraints that could result in test failures. Some examples of possible environmental issues are customizations that have been made within GitHub which change order dependencies or required data.

```bash
node utils/testRunner
answer yes at the first prompt
answer all other questions on connectivity and credentials
```

Test should also be written to clean up after themselves. However, it is important to understand that in some cases this may not be possible. In addition, whenever exceptions occur, test execution may be stopped, which will prevent cleanup actions from running. It is recommended that tests be utilized in dev and test labs only.

> **Reminder**: Do not check in code with actual credentials to systems.

## Configuration

This section defines **all** the properties that are available for the adapter, including detailed information on what each property is for. If you are not using certain capabilities with this adapter, you do not need to define all of the properties. An example of how the properties for this adapter can be used with tests or IAP are provided in the sampleProperties.

### Example Properties

```json
  "properties": {
    "host": "api.github.com",
    "port": 443,
    "choosepath": "",
    "base_path": "",
    "version": "",
    "cache_location": "none",
    "encode_pathvars": true,
    "encode_queryvars": true,
    "save_metric": false,
    "stub": true,
    "protocol": "https",
    "authentication": {
      "auth_method": "basic user_password",
      "username": "username",
      "password": "access_token",
      "token": "statictoken",
      "token_timeout": 1800000,
      "token_cache": "local",
      "invalid_token_error": 401,
      "auth_field": "header.headers.Authorization",
      "auth_field_format": "Basic {b64}{username}:{password}{/b64}",
      "auth_logging": false,
      "client_id": "",
      "client_secret": "",
      "grant_type": "",
      "sensitive": [],
      "sso": {
        "protocol": "",
        "host": "",
        "port": 0
      },
      "multiStepAuthCalls": [
        {
          "name": "",
          "requestFields": {},
          "responseFields": {},
          "successfullResponseCode": 200
        }
      ]
    },
    "healthcheck": {
      "type": "startup",
      "frequency": 300000,
      "query_object": {},
      "addlHeaders": {}
    },
    "throttle": {
      "throttle_enabled": false,
      "number_pronghorns": 1,
      "sync_async": "sync",
      "max_in_queue": 1000,
      "concurrent_max": 1,
      "expire_timeout": 0,
      "avg_runtime": 200,
      "priorities": [
        {
          "value": 0,
          "percent": 100
        }
      ]
    },
    "request": {
      "number_redirects": 0,
      "number_retries": 3,
      "limit_retry_error": 401,
      "failover_codes": [],
      "attempt_timeout": 5000,
      "global_request": {
        "payload": {},
        "uriOptions": {},
        "addlHeaders": {
          "user-agent": "node.js"
        },
        "authData": {}
      },
      "healthcheck_on_timeout": true,
      "return_raw": false,
      "archiving": false,
      "return_request": false
    },
    "proxy": {
      "enabled": false,
      "host": "",
      "port": 1,
      "protocol": "http",
      "username": "",
      "password": ""
    },
    "ssl": {
      "ecdhCurve": "",
      "enabled": false,
      "accept_invalid_cert": true,
      "ca_file": "",
      "key_file": "",
      "cert_file": "",
      "secure_protocol": "",
      "ciphers": ""
    },
    "mongo": {
      "host": "",
      "port": 0,
      "database": "",
      "username": "",
      "password": "",
      "replSet": "",
      "db_ssl": {
        "enabled": false,
        "accept_invalid_cert": false,
        "ca_file": "",
        "key_file": "",
        "cert_file": ""
      }
    },
    "devicebroker": {
      "getDevice": [
        {
          "path": "/get/devices/{id}",
          "method": "GET",
          "query": {},
          "body": {},
          "headers": {},
          "handleFailure": "ignore",
          "requestFields": {
            "id": "name"
          },
          "responseDatakey": "",
          "responseFields": {
            "name": "host",
            "ostype": "os",
            "ostypePrefix": "system-",
            "ipaddress": "attributes.ipaddr",
            "port": "443"
          }
        }
      ],
      "getDevicesFiltered": [
        {
          "path": "/get/devices",
          "method": "GET",
          "pagination": {
            "offsetVar": "",
            "limitVar": "",
            "incrementBy": "limit",
            "requestLocation": "query"
          },
          "query": {},
          "body": {},
          "headers": {},
          "handleFailure": "ignore",
          "requestFields": {},
          "responseDatakey": "",
          "responseFields": {
            "name": "host",
            "ostype": "os",
            "ostypePrefix": "system-",
            "ipaddress": "attributes.ipaddr",
            "port": "443"
          }
        }
      ],
      "isAlive": [
        {
          "path": "/get/devices/{id}/status",
          "method": "GET",
          "query": {},
          "body": {},
          "headers": {},
          "handleFailure": "ignore",
          "requestFields": {
            "id": "name"
          },
          "responseDatakey": "",
          "responseFields": {
            "status": "status",
            "statusValue": "online"
          }
        }
      ],
      "getConfig": [
        {
          "path": "/get/devices/{id}/configPart1",
          "method": "GET",
          "query": {},
          "body": {},
          "headers": {},
          "handleFailure": "ignore",
          "requestFields": {
            "id": "name"
          },
          "responseDatakey": "",
          "responseFields": {}
        }
      ],
      "getCount": [
        {
          "path": "/get/devices",
          "method": "GET",
          "query": {},
          "body": {},
          "headers": {},
          "handleFailure": "ignore",
          "requestFields": {},
          "responseDatakey": "",
          "responseFields": {}
        }
      ]
    },
    "cache": {
      "enabled": false,
      "entities": [
        {
          "entityType": "",
          "frequency": 1440,
          "flushOnFail": false,
          "limit": 1000,
          "retryAttempts": 5,
          "sort": true,
          "populate": [
            {
              "path": "",
              "method": "GET",
              "pagination": {
                "offsetVar": "",
                "limitVar": "",
                "incrementBy": "limit",
                "requestLocation": "query"
              },
              "query": {},
              "body": {},
              "headers": {},
              "handleFailure": "ignore",
              "requestFields": {},
              "responseDatakey": "",
              "responseFields": {}
            }
          ],
          "cachedTasks": [
            {
              "name": "",
              "filterField": "",
              "filterLoc": ""
            }
          ]
        }
      ]
    }
  }
```

### Connection Properties

These base properties are used to connect to GitHub upon the adapter initially coming up. It is important to set these properties appropriately.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Property</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
  </tr>
  <tr>
    <td style="padding:15px">host</td>
    <td style="padding:15px">Required. A fully qualified domain name or IP address.</td>
  </tr>
  <tr>
    <td style="padding:15px">port</td>
    <td style="padding:15px">Required. Used to connect to the server.</td>
  </tr>
  <tr>
    <td style="padding:15px">base_path</td>
    <td style="padding:15px">Optional. Used to define part of a path that is consistent for all or most endpoints. It makes the URIs easier to use and maintain but can be overridden on individual calls. An example **base_path** might be `/rest/api`. Default is ``.</td>
  </tr>
  <tr>
    <td style="padding:15px">version</td>
    <td style="padding:15px">Optional. Used to set a global version for action endpoints. This makes it faster to update the adapter when endpoints change. As with the base-path, version can be overridden on individual endpoints. Default is ``.</td>
  </tr>
  <tr>
    <td style="padding:15px">cache_location</td>
    <td style="padding:15px">Optional. Used to define where the adapter cache is located. The cache is used to maintain an entity list to improve performance. Storage locally is lost when the adapter is restarted. Storage in Redis is preserved upon adapter restart. Default is none which means no caching of the entity list.</td>
  </tr>
  <tr>
    <td style="padding:15px">encode_pathvars</td>
    <td style="padding:15px">Optional. Used to tell the adapter to encode path variables or not. The default behavior is to encode them so this property can be used to stop that behavior.</td>
  </tr>
  <tr>
    <td style="padding:15px">encode_queryvars</td>
    <td style="padding:15px">Optional. Used to tell the adapter to encode query parameters or not. The default behavior is to encode them so this property can be used to stop that behavior.</td>
  </tr>
  <tr>
    <td style="padding:15px">save_metric</td>
    <td style="padding:15px">Optional. Used to tell the adapter to save metric information (this does not impact metrics returned on calls). This allows the adapter to gather metrics over time. Metric data can be stored in a database or on the file system.</td>
  </tr>
  <tr>
    <td style="padding:15px">stub</td>
    <td style="padding:15px">Optional. Indicates whether the stub should run instead of making calls to GitHub (very useful during basic testing). Default is false (which means connect to GitHub).</td>
  </tr>
  <tr>
    <td style="padding:15px">protocol</td>
    <td style="padding:15px">Optional. Notifies the adapter whether to use HTTP or HTTPS. Default is HTTP.</td>
  </tr>
</table>
<br>

A connectivity check tells IAP the adapter has loaded successfully.

### Authentication Properties

The following properties are used to define the authentication process to GitHub.

>**Note**: Depending on the method that is used to authenticate with GitHub, you may not need to set all of the authentication properties.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Property</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
  </tr>
  <tr>
    <td style="padding:15px">auth_method</td>
    <td style="padding:15px">Required. Used to define the type of authentication currently supported. Authentication methods currently supported are: `basic user_password`, `static_token`, `request_token`, and `no_authentication`.</td>
  </tr>
  <tr>
    <td style="padding:15px">username</td>
    <td style="padding:15px">Used to authenticate with GitHub on every request or when pulling a token that will be used in subsequent requests.</td>
  </tr>
  <tr>
    <td style="padding:15px">password</td>
    <td style="padding:15px">Used to authenticate with GitHub on every request or when pulling a token that will be used in subsequent requests.</td>
  </tr>
  <tr>
    <td style="padding:15px">token</td>
    <td style="padding:15px">Defines a static token that can be used on all requests. Only used with `static_token` as an authentication method (auth\_method).</td>
  </tr>
  <tr>
    <td style="padding:15px">invalid_token_error</td>
    <td style="padding:15px">Defines the HTTP error that is received when the token is invalid. Notifies the adapter to pull a new token and retry the request. Default is 401.</td>
  </tr>
  <tr>
    <td style="padding:15px">token_timeout</td>
    <td style="padding:15px">Defines how long a token is valid. Measured in milliseconds. Once a dynamic token is no longer valid, the adapter has to pull a new token. If the token_timeout is set to -1, the adapter will pull a token on every request to GitHub. If the timeout_token is 0, the adapter will use the expiration from the token response to determine when the token is no longer valid.</td>
  </tr>
  <tr>
    <td style="padding:15px">token_cache</td>
    <td style="padding:15px">Used to determine where the token should be stored (local memory or in Redis).</td>
  </tr>
  <tr>
    <td style="padding:15px">auth_field</td>
    <td style="padding:15px">Defines the request field the authentication (e.g., token are basic auth credentials) needs to be placed in order for the calls to work.</td>
  </tr>
  <tr>
    <td style="padding:15px">auth_field_format</td>
    <td style="padding:15px">Defines the format of the auth\_field. See examples below. Items enclosed in {} inform the adapter to perofrm an action prior to sending the data. It may be to replace the item with a value or it may be to encode the item.</td>
  </tr>
  <tr>
    <td style="padding:15px">auth_logging</td>
    <td style="padding:15px">Setting this true will add some additional logs but this should only be done when trying to debug an issue as certain credential information may be logged out when this is true.</td>
  </tr>
  <tr>
    <td style="padding:15px">client_id</td>
    <td style="padding:15px">Provide a client id when needed, this is common on some types of OAuth.</td>
  </tr>
  <tr>
    <td style="padding:15px">client_secret</td>
    <td style="padding:15px">Provide a client secret when needed, this is common on some types of OAuth.</td>
  </tr>
  <tr>
    <td style="padding:15px">grant_type</td>
    <td style="padding:15px">Provide a grant type when needed, this is common on some types of OAuth.</td>
  </tr>
</table>
<br>

#### Examples of authentication field format

```json
"{token}"
"Token {token}"
"{username}:{password}"
"Basic {b64}{username}:{password}{/b64}"
```

### Healthcheck Properties

The healthcheck properties defines the API that runs the healthcheck to tell the adapter that it can reach GitHub. There are currently three types of healthchecks.

- None - Not recommended. Adapter will not run a healthcheck. Consequently, unable to determine before making a request if the adapter can reach GitHub.
- Startup - Adapter will check for connectivity when the adapter initially comes up, but it will not check afterwards.
- Intermittent - Adapter will check connectivity to GitHub at a frequency defined in the `frequency` property.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Property</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
  </tr>
  <tr>
    <td style="padding:15px">type</td>
    <td style="padding:15px">Required. The type of health check to run.</td>
  </tr>
  <tr>
    <td style="padding:15px">frequency</td>
    <td style="padding:15px">Required if intermittent. Defines how often the health check should run. Measured in milliseconds. Default is 300000.</td>
  </tr>
  <tr>
    <td style="padding:15px">query_object</td>
    <td style="padding:15px">Query parameters to be added to the adapter healthcheck call.</td>
  </tr>
</table>
<br>

### Request Properties

The request section defines properties to help handle requests.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Property</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
  </tr>
  <tr>
    <td style="padding:15px">number_redirects</td>
    <td style="padding:15px">Optional. Tells the adapter that the request may be redirected and gives it a maximum number of redirects to allow before returning an error. Default is 0 - no redirects.</td>
  </tr>
  <tr>
    <td style="padding:15px">number_retries</td>
    <td style="padding:15px">Tells the adapter how many times to retry a request that has either aborted or reached a limit error before giving up and returning an error.</td>
  </tr>
  <tr>
    <td style="padding:15px">limit_retry_error</td>
    <td style="padding:15px">Optional. Can be either an integer or an array. Indicates the http error status number to define that no capacity was available and, after waiting a short interval, the adapter can retry the request. If an array is provvided, the array can contain integers or strings. Strings in the array are used to define ranges (e.g. "502-506"). Default is [0].</td>
  </tr>
  <tr>
    <td style="padding:15px">failover_codes</td>
    <td style="padding:15px">An array of error codes for which the adapter will send back a failover flag to IAP so that the Platform can attempt the action in another adapter.</td>
  </tr>
  <tr>
    <td style="padding:15px">attempt_timeout</td>
    <td style="padding:15px">Optional. Tells how long the adapter should wait before aborting the attempt. On abort, the adapter will do one of two things: 1) return the error; or 2) if **healthcheck\_on\_timeout** is set to true, it will abort the request and run a Healthcheck until it re-establishes connectivity to GitHub, and then will re-attempt the request that aborted. Default is 5000 milliseconds.</td>
  </tr>
  <tr>
    <td style="padding:15px">global_request</td>
    <td style="padding:15px">Optional. This is information that the adapter can include in all requests to the other system. This is easier to define and maintain than adding this information in either the code (adapter.js) or the action files.</td>
  </tr>
  <tr>
    <td style="padding:15px">global_request -> payload</td>
    <td style="padding:15px">Optional. Defines any information that should be included on all requests sent to the other system that have a payload/body.</td>
  </tr>
  <tr>
    <td style="padding:15px">global_request -> uriOptions</td>
    <td style="padding:15px">Optional. Defines any information that should be sent as untranslated  query options (e.g. page, size) on all requests to the other system.</td>
  </tr>
  <tr>
    <td style="padding:15px">global_request -> addlHeaders</td>
    <td style="padding:15px">Optioonal. Defines any headers that should be sent on all requests to the other system.</td>
  </tr>
  <tr>
    <td style="padding:15px">global_request -> authData</td>
    <td style="padding:15px">Optional. Defines any additional authentication data used to authentice with the other system. This authData needs to be consistent on every request.</td>
  </tr>
  <tr>
    <td style="padding:15px">healthcheck_on_timeout</td>
    <td style="padding:15px">Required. Defines if the adapter should run a health check on timeout. If set to true, the adapter will abort the request and run a health check until it re-establishes connectivity and then it will re-attempt the request.</td>
  </tr>
  <tr>
    <td style="padding:15px">return_raw</td>
    <td style="padding:15px">Optional. Tells the adapter whether the raw response should be returned as well as the IAP response. This is helpful when running integration tests to save mock data. It does add overhead to the response object so it is not ideal from production.</td>
  </tr>
  <tr>
    <td style="padding:15px">archiving</td>
    <td style="padding:15px">Optional flag. Default is false. It archives the request, the results and the various times (wait time, GitHub time and overall time) in the `adapterid_results` collection in MongoDB. Although archiving might be desirable, be sure to develop a strategy before enabling this capability. Consider how much to archive and what strategy to use for cleaning up the collection in the database so that it does not become too large, especially if the responses are large.</td>
  </tr>
  <tr>
    <td style="padding:15px">return_request</td>
    <td style="padding:15px">Optional flag. Default is false. Will return the actual request that is made including headers. This should only be used during debugging issues as there could be credentials in the actual request.</td>
  </tr>
</table>
<br>

### SSL Properties

The SSL section defines the properties utilized for ssl authentication with GitHub. SSL can work two different ways: set the `accept\_invalid\_certs` flag to true (only recommended for lab environments), or provide a `ca\_file`.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Property</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
  </tr>
  <tr>
    <td style="padding:15px">enabled</td>
    <td style="padding:15px">If SSL is required, set to true.</td>
  </tr>
  <tr>
    <td style="padding:15px">accept_invalid_certs</td>
    <td style="padding:15px">Defines if the adapter should accept invalid certificates (only recommended for lab environments). Required if SSL is enabled. Default is false.</td>
  </tr>
  <tr>
    <td style="padding:15px">ca_file</td>
    <td style="padding:15px">Defines the path name to the CA file used for SSL. If SSL is enabled and the accept invalid certifications is false, then ca_file is required.</td>
  </tr>
  <tr>
    <td style="padding:15px">key_file</td>
    <td style="padding:15px">Defines the path name to the Key file used for SSL. The key_file may be needed for some systems but it is not required for SSL.</td>
  </tr>
  <tr>
    <td style="padding:15px">cert_file</td>
    <td style="padding:15px">Defines the path name to the Certificate file used for SSL. The cert_file may be needed for some systems but it is not required for SSL.</td>
  </tr>
  <tr>
    <td style="padding:15px">secure_protocol</td>
    <td style="padding:15px">Defines the protocol (e.g., SSLv3_method) to use on the SSL request.</td>
  </tr>
  <tr>
    <td style="padding:15px">ciphers</td>
    <td style="padding:15px">Required if SSL enabled. Specifies a list of SSL ciphers to use.</td>
  </tr>
  <tr>
    <td style="padding:15px">ecdhCurve</td>
    <td style="padding:15px">During testing on some Node 8 environments, you need to set `ecdhCurve` to auto. If you do not, you will receive PROTO errors when attempting the calls. This is the only usage of this property and to our knowledge it only impacts Node 8 and 9.</td>
  </tr>
</table>
<br>

### Throttle Properties

The throttle section is used when requests to GitHub must be queued (throttled). All of the properties in this section are optional.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Property</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
  </tr>
  <tr>
    <td style="padding:15px">throttle_enabled</td>
    <td style="padding:15px">Default is false. Defines if the adapter should use throttling or not.</td>
  </tr>
  <tr>
    <td style="padding:15px">number_pronghorns</td>
    <td style="padding:15px">Default is 1. Defines if throttling is done in a single Itential instance or whether requests are being throttled across multiple Itential instances (minimum = 1, maximum = 20). Throttling in a single Itential instance uses an in-memory queue so there is less overhead. Throttling across multiple Itential instances requires placing the request and queue information into a shared resource (e.g. database) so that each instance can determine what is running and what is next to run. Throttling across multiple instances requires additional I/O overhead.</td>
  </tr>
  <tr>
    <td style="padding:15px">sync-async</td>
    <td style="padding:15px">This property is not used at the current time (it is for future expansion of the throttling engine).</td>
  </tr>
  <tr>
    <td style="padding:15px">max_in_queue</td>
    <td style="padding:15px">Represents the maximum number of requests the adapter should allow into the queue before rejecting requests (minimum = 1, maximum = 5000). This is not a limit on what the adapter can handle but more about timely responses to requests. The default is currently 1000.</td>
  </tr>
  <tr>
    <td style="padding:15px">concurrent_max</td>
    <td style="padding:15px">Defines the number of requests the adapter can send to GitHub at one time (minimum = 1, maximum = 1000). The default is 1 meaning each request must be sent to GitHub in a serial manner.</td>
  </tr>
  <tr>
    <td style="padding:15px">expire_timeout</td>
    <td style="padding:15px">Default is 0. Defines a graceful timeout of the request session. After a request has completed, the adapter will wait additional time prior to sending the next request. Measured in milliseconds (minimum = 0, maximum = 60000).</td>
  </tr>
  <tr>
    <td style="padding:15px">average_runtime</td>
    <td style="padding:15px">Represents the approximate average of how long it takes GitHub to handle each request. Measured in milliseconds (minimum = 50, maximum = 60000). Default is 200. This metric has performance implications. If the runtime number is set too low, it puts extra burden on the CPU and memory as the requests will continually try to run. If the runtime number is set too high, requests may wait longer than they need to before running. The number does not need to be exact but your throttling strategy depends heavily on this number being within reason. If averages range from 50 to 250 milliseconds you might pick an average run-time somewhere in the middle so that when GitHub performance is exceptional you might run a little slower than you might like, but when it is poor you still run efficiently.</td>
  </tr>
  <tr>
    <td style="padding:15px">priorities</td>
    <td style="padding:15px">An array of priorities and how to handle them in relation to the throttle queue. Array of objects that include priority value and percent of queue to put the item ex { value: 1, percent: 10 }</td>
  </tr>
</table>
<br>

### Proxy Properties

The proxy section defines the properties to utilize when GitHub is behind a proxy server.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Property</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
  </tr>
  <tr>
    <td style="padding:15px">enabled</td>
    <td style="padding:15px">Required. Default is false. If GitHub is behind a proxy server, set enabled flag to true.</td>
  </tr>
  <tr>
    <td style="padding:15px">host</td>
    <td style="padding:15px">Host information for the proxy server. Required if `enabled` is true.</td>
  </tr>
  <tr>
    <td style="padding:15px">port</td>
    <td style="padding:15px">Port information for the proxy server. Required if `enabled` is true.</td>
  </tr>
  <tr>
    <td style="padding:15px">protocol</td>
    <td style="padding:15px">The protocol (i.e., http, https, etc.) used to connect to the proxy. Default is http.</td>
  </tr>
  <tr>
    <td style="padding:15px">username</td>
    <td style="padding:15px">If there is authentication for the proxy, provide the username here.</td>
  </tr>
  <tr>
    <td style="padding:15px">password</td>
    <td style="padding:15px">If there is authentication for the proxy, provide the password here.</td>
  </tr>
</table>
<br>

### Mongo Properties

The mongo section defines the properties used to connect to a Mongo database. Mongo can be used for throttling as well as to persist metric data. If not provided, metrics will be stored in the file system.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Property</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
  </tr>
  <tr>
    <td style="padding:15px">host</td>
    <td style="padding:15px">Optional. Host information for the mongo server.</td>
  </tr>
  <tr>
    <td style="padding:15px">port</td>
    <td style="padding:15px">Optional. Port information for the mongo server.</td>
  </tr>
  <tr>
    <td style="padding:15px">database</td>
    <td style="padding:15px">Optional. The database for the adapter to use for its data.</td>
  </tr>
  <tr>
    <td style="padding:15px">username</td>
    <td style="padding:15px">Optional. If credentials are required to access mongo, this is the user to login as.</td>
  </tr>
  <tr>
    <td style="padding:15px">password</td>
    <td style="padding:15px">Optional. If credentials are required to access mongo, this is the password to login with.</td>
  </tr>
  <tr>
    <td style="padding:15px">replSet</td>
    <td style="padding:15px">Optional. If the database is set up to use replica sets, define it here so it can be added to the database connection.</td>
  </tr>
  <tr>
    <td style="padding:15px">db_ssl</td>
    <td style="padding:15px">Optional. Contains information for SSL connectivity to the database.</td>
  </tr>
  <tr>
    <td style="padding:15px">db_ssl -> enabled</td>
    <td style="padding:15px">If SSL is required, set to true.</td>
  </tr>
  <tr>
    <td style="padding:15px">db_ssl -> accept_invalid_cert</td>
    <td style="padding:15px">Defines if the adapter should accept invalid certificates (only recommended for lab environments). Required if SSL is enabled. Default is false.</td>
  </tr>
  <tr>
    <td style="padding:15px">db_ssl -> ca_file</td>
    <td style="padding:15px">Defines the path name to the CA file used for SSL. If SSL is enabled and the accept invalid certifications is false, then ca_file is required.</td>
  </tr>
  <tr>
    <td style="padding:15px">db_ssl -> key_file</td>
    <td style="padding:15px">Defines the path name to the Key file used for SSL. The key_file may be needed for some systems but it is not required for SSL.</td>
  </tr>
  <tr>
    <td style="padding:15px">db_ssl -> cert_file</td>
    <td style="padding:15px">Defines the path name to the Certificate file used for SSL. The cert_file may be needed for some systems but it is not required for SSL.</td>
  </tr>
</table>
<br>

### Device Broker Properties

The device broker section defines the properties used integrate GitHub to the device broker. Each broker call is represented and has an array of calls that can be used to build the response. This describes the calls and then the fields which are available in the calls.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Property</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
  </tr>
  <tr>
    <td style="padding:15px">getDevice</td>
    <td style="padding:15px">The array of calls used to get device details for the broker</td>
  </tr>
  <tr>
    <td style="padding:15px">getDevicesFiltered</td>
    <td style="padding:15px">The array of calls used to get devices for the broker</td>
  </tr>
  <tr>
    <td style="padding:15px">isAlive</td>
    <td style="padding:15px">The array of calls used to get device status for the broker</td>
  </tr>
  <tr>
    <td style="padding:15px">getConfig</td>
    <td style="padding:15px">The array of calls used to get device configuration for the broker</td>
  </tr>
  <tr>
    <td style="padding:15px">getCount</td>
    <td style="padding:15px">The array of calls used to get device configuration for the broker</td>
  </tr>
  <tr>
    <td style="padding:15px">getDevice/getDevicesFiltered/isAlive/getConfig/getCount -> path</td>
    <td style="padding:15px">The path, not including the base_path and version, for making this call</td>
  </tr>
  <tr>
    <td style="padding:15px">getDevice/getDevicesFiltered/isAlive/getConfig/getCount -> method</td>
    <td style="padding:15px">The rest method for making this call</td>
  </tr>
  <tr>
    <td style="padding:15px">getDevice/getDevicesFiltered/isAlive/getConfig/getCount -> query</td>
    <td style="padding:15px">Query object containing and query parameters and their values for this call</td>
  </tr>
  <tr>
    <td style="padding:15px">getDevice/getDevicesFiltered/isAlive/getConfig/getCount -> body</td>
    <td style="padding:15px">Body object containing the payload for this call</td>
  </tr>
  <tr>
    <td style="padding:15px">getDevice/getDevicesFiltered/isAlive/getConfig/getCount -> headers</td>
    <td style="padding:15px">Header object containing the headers for this call.</td>
  </tr>
  <tr>
    <td style="padding:15px">getDevice/getDevicesFiltered/isAlive/getConfig/getCount -> handleFailure</td>
    <td style="padding:15px">Tells the adapter whether to "fail" or "ignore" failures if they occur.</td>
  </tr>
  <tr>
    <td style="padding:15px">isAlive -> statusValue</td>
    <td style="padding:15px">Tells the adapter what value to look for in the status field to determine if the device is alive.</td>
  </tr>
  <tr>
    <td style="padding:15px">getDevice/getDevicesFiltered/isAlive/getConfig -> requestFields</td>
    <td style="padding:15px">Object containing fields the adapter should send on the request and where it should get the data. The where can be from a response to a getDevicesFiltered or a static value.</td>
  </tr>
  <tr>
    <td style="padding:15px">getDevice/getDevicesFiltered/isAlive/getConfig -> responseFields</td>
    <td style="padding:15px">Object containing fields the adapter should set to send back to iap and where the value should come from in the response or request data.</td>
  </tr>
</table>
<br>


## Using this Adapter

The `adapter.js` file contains the calls the adapter makes available to the rest of the Itential Platform. The API detailed for these calls should be available through JSDOC. The following is a brief summary of the calls.

### Generic Adapter Calls

These are adapter methods that IAP or you might use. There are some other methods not shown here that might be used for internal adapter functionality.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Method Signature</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Workflow?</span></th>
  </tr>
  <tr>
    <td style="padding:15px">connect()</td>
    <td style="padding:15px">This call is run when the Adapter is first loaded by he Itential Platform. It validates the properties have been provided correctly.</td>
    <td style="padding:15px">No</td>
  </tr>
  <tr>
    <td style="padding:15px">healthCheck(callback)</td>
    <td style="padding:15px">This call ensures that the adapter can communicate with Adapter for GitHub System. The actual call that is used is defined in the adapter properties and .system entities action.json file.</td>
    <td style="padding:15px">No</td>
  </tr>
  <tr>
    <td style="padding:15px">refreshProperties(properties)</td>
    <td style="padding:15px">This call provides the adapter the ability to accept property changes without having to restart the adapter.</td>
    <td style="padding:15px">No</td>
  </tr>
  <tr>
    <td style="padding:15px">encryptProperty(property, technique, callback)</td>
    <td style="padding:15px">This call will take the provided property and technique, and return the property encrypted with the technique. This allows the property to be used in the adapterProps section for the credential password so that the password does not have to be in clear text. The adapter will decrypt the property as needed for communications with Adapter for GitHub System.</td>
    <td style="padding:15px">No</td>
  </tr>
  <tr>
    <td style="padding:15px">iapUpdateAdapterConfiguration(configFile, changes, entity, type, action, callback)</td>
    <td style="padding:15px">This call provides the ability to update the adapter configuration from IAP - includes actions, schema, mockdata and other configurations.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapSuspendAdapter(mode, callback)</td>
    <td style="padding:15px">This call provides the ability to suspend the adapter and either have requests rejected or put into a queue to be processed after the adapter is resumed.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapUnsuspendAdapter(callback)</td>
    <td style="padding:15px">This call provides the ability to resume a suspended adapter. Any requests in queue will be processed before new requests.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapGetAdapterQueue(callback)</td>
    <td style="padding:15px">This call will return the requests that are waiting in the queue if throttling is enabled.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapFindAdapterPath(apiPath, callback)</td>
    <td style="padding:15px">This call provides the ability to see if a particular API path is supported by the adapter.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapTroubleshootAdapter(props, persistFlag, adapter, callback)</td>
    <td style="padding:15px">This call can be used to check on the performance of the adapter - it checks connectivity, healthcheck and basic get calls.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapRunAdapterHealthcheck(adapter, callback)</td>
    <td style="padding:15px">This call will return the results of a healthcheck.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapRunAdapterConnectivity(callback)</td>
    <td style="padding:15px">This call will return the results of a connectivity check.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapRunAdapterBasicGet(callback)</td>
    <td style="padding:15px">This call will return the results of running basic get API calls.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapMoveAdapterEntitiesToDB(callback)</td>
    <td style="padding:15px">This call will push the adapter configuration from the entities directory into the Adapter or IAP Database.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapDeactivateTasks(tasks, callback)</td>
    <td style="padding:15px">This call provides the ability to remove tasks from the adapter.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapActivateTasks(tasks, callback)</td>
    <td style="padding:15px">This call provides the ability to add deactivated tasks back into the adapter.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapExpandedGenericAdapterRequest(metadata, uriPath, restMethod, pathVars, queryData, requestBody, addlHeaders, callback)</td>
    <td style="padding:15px">This is an expanded Generic Call. The metadata object allows us to provide many new capabilities within the generic request.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">genericAdapterRequest(uriPath, restMethod, queryData, requestBody, addlHeaders, callback)</td>
    <td style="padding:15px">This call allows you to provide the path to have the adapter call. It is an easy way to incorporate paths that have not been built into the adapter yet.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">genericAdapterRequestNoBasePath(uriPath, restMethod, queryData, requestBody, addlHeaders, callback)</td>
    <td style="padding:15px">This call is the same as the genericAdapterRequest only it does not add a base_path or version to the call.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapRunAdapterLint(callback)</td>
    <td style="padding:15px">Runs lint on the addapter and provides the information back.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapRunAdapterTests(callback)</td>
    <td style="padding:15px">Runs baseunit and unit tests on the adapter and provides the information back.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapGetAdapterInventory(callback)</td>
    <td style="padding:15px">This call provides some inventory related information about the adapter.</td>
    <td style="padding:15px">Yes</td>
  </tr>
</table>
<br>
  
### Adapter Cache Calls

These are adapter methods that are used for adapter caching. If configured, the adapter will cache based on the interval provided. However, you can force a population of the cache manually as well.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Method Signature</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Workflow?</span></th>
  </tr>
  <tr>
    <td style="padding:15px">iapPopulateEntityCache(entityTypes, callback)</td>
    <td style="padding:15px">This call populates the adapter cache.</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">iapRetrieveEntitiesCache(entityType, options, callback)</td>
    <td style="padding:15px">This call retrieves the specific items from the adapter cache.</td>
    <td style="padding:15px">Yes</td>
  </tr>
</table>
<br>
  
### Adapter Broker Calls

These are adapter methods that are used to integrate to IAP Brokers. This adapter currently supports the following broker calls.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Method Signature</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Workflow?</span></th>
  </tr>
  <tr>
    <td style="padding:15px">hasEntities(entityType, entityList, callback)</td>
    <td style="padding:15px">This call is utilized by the IAP Device Broker to determine if the adapter has a specific entity and item of the entity.</td>
    <td style="padding:15px">No</td>
  </tr>
  <tr>
    <td style="padding:15px">getDevice(deviceName, callback)</td>
    <td style="padding:15px">This call returns the details of the requested device.</td>
    <td style="padding:15px">No</td>
  </tr>
  <tr>
    <td style="padding:15px">getDevicesFiltered(options, callback)</td>
    <td style="padding:15px">This call returns the list of devices that match the criteria provided in the options filter.</td>
    <td style="padding:15px">No</td>
  </tr>
  <tr>
    <td style="padding:15px">isAlive(deviceName, callback)</td>
    <td style="padding:15px">This call returns whether the device status is active</td>
    <td style="padding:15px">No</td>
  </tr>
  <tr>
    <td style="padding:15px">getConfig(deviceName, format, callback)</td>
    <td style="padding:15px">This call returns the configuration for the selected device.</td>
    <td style="padding:15px">No</td>
  </tr>
  <tr>
    <td style="padding:15px">iapGetDeviceCount(callback)</td>
    <td style="padding:15px">This call returns the count of devices.</td>
    <td style="padding:15px">No</td>
  </tr>
</table>
<br>

### Specific Adapter Calls

Specific adapter calls are built based on the API of the GitHub System. The Adapter Builder creates the proper method comments for generating JS-DOC for the adapter. This is the best way to get information on the calls.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Method Signature</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Path</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Workflow?</span></th>
  </tr>
  <tr>
    <td style="padding:15px">getEmojis(callback)</td>
    <td style="padding:15px">Lists all the emojis available to use on GitHub.</td>
    <td style="padding:15px">{base_path}/{version}/emojis?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getEvents(callback)</td>
    <td style="padding:15px">List public events.</td>
    <td style="padding:15px">{base_path}/{version}/events?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getFeeds(callback)</td>
    <td style="padding:15px">List Feeds.
GitHub provides several timeline resources in Atom format. The Feeds API
 lists all the feeds available to the authenticating user.</td>
    <td style="padding:15px">{base_path}/{version}/feeds?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getGists(since, callback)</td>
    <td style="padding:15px">List the authenticated user's gists or if called anonymously, this will
return all public gists.</td>
    <td style="padding:15px">{base_path}/{version}/gists?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postGists(body, callback)</td>
    <td style="padding:15px">Create a gist.</td>
    <td style="padding:15px">{base_path}/{version}/gists?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getGistsPublic(since, callback)</td>
    <td style="padding:15px">List all public gists.</td>
    <td style="padding:15px">{base_path}/{version}/gists/public?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getGistsStarred(since, callback)</td>
    <td style="padding:15px">List the authenticated user's starred gists.</td>
    <td style="padding:15px">{base_path}/{version}/gists/starred?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteGistsId(id, callback)</td>
    <td style="padding:15px">Delete a gist.</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getGistsId(id, callback)</td>
    <td style="padding:15px">Get a single gist.</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchGistsId(id, body, callback)</td>
    <td style="padding:15px">Edit a gist.</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getGistsIdComments(id, callback)</td>
    <td style="padding:15px">List comments on a gist.</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}/comments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postGistsIdComments(id, body, callback)</td>
    <td style="padding:15px">Create a commen</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}/comments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteGistsIdCommentsCommentId(id, commentId, callback)</td>
    <td style="padding:15px">Delete a comment.</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}/comments/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getGistsIdCommentsCommentId(id, commentId, callback)</td>
    <td style="padding:15px">Get a single comment.</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}/comments/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchGistsIdCommentsCommentId(id, commentId, body, callback)</td>
    <td style="padding:15px">Edit a comment.</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}/comments/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postGistsIdForks(id, callback)</td>
    <td style="padding:15px">Fork a gist.</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}/forks?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteGistsIdStar(id, callback)</td>
    <td style="padding:15px">Unstar a gist.</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}/star?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getGistsIdStar(id, callback)</td>
    <td style="padding:15px">Check if a gist is starred.</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}/star?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putGistsIdStar(id, callback)</td>
    <td style="padding:15px">Star a gist.</td>
    <td style="padding:15px">{base_path}/{version}/gists/{pathv1}/star?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getGitignoreTemplates(callback)</td>
    <td style="padding:15px">Listing available templates.
List all templates available to pass as an option when creating a repository.</td>
    <td style="padding:15px">{base_path}/{version}/gitignore/templates?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getGitignoreTemplatesLanguage(language, callback)</td>
    <td style="padding:15px">Get a single template.</td>
    <td style="padding:15px">{base_path}/{version}/gitignore/templates/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getIssues(filter = 'assigned', state = 'open', labels, sort = 'created', direction = 'asc', since, callback)</td>
    <td style="padding:15px">List issues.
List all issues across all the authenticated user's visible repositories.</td>
    <td style="padding:15px">{base_path}/{version}/issues?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getLegacyIssuesSearchOwnerRepositoryStateKeyword(keyword, state = 'open', owner, repository, callback)</td>
    <td style="padding:15px">Find issues by state and keyword.</td>
    <td style="padding:15px">{base_path}/{version}/legacy/issues/search/{pathv1}/{pathv2}/{pathv3}/{pathv4}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getLegacyReposSearchKeyword(keyword, order = 'desc', language, startPage, sort = 'updated', callback)</td>
    <td style="padding:15px">Find repositories by keyword. Note, this legacy method does not follow the v3 pagination pattern. This method returns up to 100 results per page and pages can be fetched using the start_page parameter.</td>
    <td style="padding:15px">{base_path}/{version}/legacy/repos/search/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getLegacyUserEmailEmail(email, callback)</td>
    <td style="padding:15px">This API call is added for compatibility reasons only.</td>
    <td style="padding:15px">{base_path}/{version}/legacy/user/email/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getLegacyUserSearchKeyword(keyword, order = 'desc', startPage, sort = 'updated', callback)</td>
    <td style="padding:15px">Find users by keyword.</td>
    <td style="padding:15px">{base_path}/{version}/legacy/user/search/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postMarkdown(body, callback)</td>
    <td style="padding:15px">Render an arbitrary Markdown document</td>
    <td style="padding:15px">{base_path}/{version}/markdown?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postMarkdownRaw(callback)</td>
    <td style="padding:15px">Render a Markdown document in raw mode</td>
    <td style="padding:15px">{base_path}/{version}/markdown/raw?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getMeta(callback)</td>
    <td style="padding:15px">This gives some information about GitHub.com, the service.</td>
    <td style="padding:15px">{base_path}/{version}/meta?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getNetworksOwnerRepoEvents(owner, repo, callback)</td>
    <td style="padding:15px">List public events for a network of repositories.</td>
    <td style="padding:15px">{base_path}/{version}/networks/{pathv1}/{pathv2}/events?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getNotifications(all, participating, since, callback)</td>
    <td style="padding:15px">List your notifications.
List all notifications for the current user, grouped by repository.</td>
    <td style="padding:15px">{base_path}/{version}/notifications?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putNotifications(body, callback)</td>
    <td style="padding:15px">Mark as read.
Marking a notification as "read" removes it from the default view on GitHub.com.</td>
    <td style="padding:15px">{base_path}/{version}/notifications?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getNotificationsThreadsId(id, callback)</td>
    <td style="padding:15px">View a single thread.</td>
    <td style="padding:15px">{base_path}/{version}/notifications/threads/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchNotificationsThreadsId(id, callback)</td>
    <td style="padding:15px">Mark a thread as read</td>
    <td style="padding:15px">{base_path}/{version}/notifications/threads/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteNotificationsThreadsIdSubscription(id, callback)</td>
    <td style="padding:15px">Delete a Thread Subscription.</td>
    <td style="padding:15px">{base_path}/{version}/notifications/threads/{pathv1}/subscription?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getNotificationsThreadsIdSubscription(id, callback)</td>
    <td style="padding:15px">Get a Thread Subscription.</td>
    <td style="padding:15px">{base_path}/{version}/notifications/threads/{pathv1}/subscription?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putNotificationsThreadsIdSubscription(id, body, callback)</td>
    <td style="padding:15px">Set a Thread Subscription.
This lets you subscribe to a thread, or ignore it. Subscribing to a thread
is unnecessary if the user is already subscribed to the repository. Ignoring
a thread will mute all future notifications (until you comment or get @mentioned).</td>
    <td style="padding:15px">{base_path}/{version}/notifications/threads/{pathv1}/subscription?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getOrgsOrg(org, callback)</td>
    <td style="padding:15px">Get an Organization.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchOrgsOrg(org, body, callback)</td>
    <td style="padding:15px">Edit an Organization.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getOrgsOrgEvents(org, callback)</td>
    <td style="padding:15px">List public events for an organization.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/events?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getOrgsOrgIssues(org, filter = 'assigned', state = 'open', labels, sort = 'created', direction = 'asc', since, callback)</td>
    <td style="padding:15px">List issues.
List all issues for a given organization for the authenticated user.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/issues?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getOrgsOrgMembers(org, callback)</td>
    <td style="padding:15px">Members list.
List all users who are members of an organization. A member is a user tha
belongs to at least 1 team in the organization. If the authenticated user
is also an owner of this organization then both concealed and public members
will be returned. If the requester is not an owner of the organization the
query will be redirected to the public members list.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/members?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteOrgsOrgMembersUsername(org, username, callback)</td>
    <td style="padding:15px">Remove a member.
Removing a user from this list will remove them from all teams and they
will no longer have any access to the organization's repositories.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/members/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getOrgsOrgMembersUsername(org, username, callback)</td>
    <td style="padding:15px">Check if a user is, publicly or privately, a member of the organization.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/members/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getOrgsOrgPublicMembers(org, callback)</td>
    <td style="padding:15px">Public members list.
Members of an organization can choose to have their membership publicized
or not.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/public_members?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteOrgsOrgPublicMembersUsername(org, username, callback)</td>
    <td style="padding:15px">Conceal a user's membership.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/public_members/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getOrgsOrgPublicMembersUsername(org, username, callback)</td>
    <td style="padding:15px">Check public membership.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/public_members/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putOrgsOrgPublicMembersUsername(org, username, callback)</td>
    <td style="padding:15px">Publicize a user's membership.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/public_members/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getOrgsOrgRepos(org, type = 'all', callback)</td>
    <td style="padding:15px">List repositories for the specified org.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/repos?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postOrgsOrgRepos(org, body, callback)</td>
    <td style="padding:15px">Create a new repository for the authenticated user. OAuth users must supply
repo scope.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/repos?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getOrgsOrgTeams(org, callback)</td>
    <td style="padding:15px">List teams.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/teams?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postOrgsOrgTeams(org, body, callback)</td>
    <td style="padding:15px">Create team.
In order to create a team, the authenticated user must be an owner of organization.</td>
    <td style="padding:15px">{base_path}/{version}/orgs/{pathv1}/teams?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getRateLimit(callback)</td>
    <td style="padding:15px">Get your current rate limit status
Note: Accessing this endpoint does not count against your rate limit.</td>
    <td style="padding:15px">{base_path}/{version}/rate_limit?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepo(owner, repo, callback)</td>
    <td style="padding:15px">Delete a Repository.
Deleting a repository requires admin access. If OAuth is used, the delete_repo
scope is required.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepo(owner, repo, callback)</td>
    <td style="padding:15px">Get repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepo(owner, repo, body, callback)</td>
    <td style="padding:15px">Edit repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoAssignees(owner, repo, callback)</td>
    <td style="padding:15px">List assignees.
This call lists all the available assignees (owner + collaborators) to which
issues may be assigned.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/assignees?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoAssigneesAssignee(owner, repo, assignee, callback)</td>
    <td style="padding:15px">Check assignee.
You may also check to see if a particular user is an assignee for a repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/assignees/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoBranches(owner, repo, callback)</td>
    <td style="padding:15px">Get list of branches</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/branches?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoBranchesBranch(owner, repo, branch, callback)</td>
    <td style="padding:15px">Get Branch</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/branches/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoCollaborators(owner, repo, callback)</td>
    <td style="padding:15px">List.
When authenticating as an organization owner of an organization-owned
repository, all organization owners are included in the list of
collaborators. Otherwise, only users with access to the repository are
returned in the collaborators list.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/collaborators?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoCollaboratorsUser(owner, repo, user, callback)</td>
    <td style="padding:15px">Remove collaborator.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/collaborators/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoCollaboratorsUser(owner, repo, user, callback)</td>
    <td style="padding:15px">Check if user is a collaborator</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/collaborators/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putReposOwnerRepoCollaboratorsUser(owner, repo, user, callback)</td>
    <td style="padding:15px">Add collaborator.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/collaborators/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoComments(owner, repo, callback)</td>
    <td style="padding:15px">List commit comments for a repository.
Comments are ordered by ascending ID.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/comments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoCommentsCommentId(owner, repo, commentId, callback)</td>
    <td style="padding:15px">Delete a commit comment</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/comments/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoCommentsCommentId(owner, repo, commentId, callback)</td>
    <td style="padding:15px">Get a single commit comment.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/comments/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepoCommentsCommentId(owner, repo, commentId, body, callback)</td>
    <td style="padding:15px">Update a commit comment.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/comments/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoCommits(owner, repo, since, sha, pathParam, author, until, callback)</td>
    <td style="padding:15px">List commits on a repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/commits?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoCommitsRefStatus(owner, repo, ref, callback)</td>
    <td style="padding:15px">Get the combined Status for a specific Ref
The Combined status endpoint is currently available for developers to preview. During the preview period, the API may change without advance notice. Please see the blog post for full details.
To access this endpoint during the preview period, you must provide a custom media type in the Accept header:
application/vnd.github.she-hulk-preview+json</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/commits/{pathv3}/status?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoCommitsShaCode(owner, repo, shaCode, callback)</td>
    <td style="padding:15px">Get a single commit.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/commits/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoCommitsShaCodeComments(owner, repo, shaCode, callback)</td>
    <td style="padding:15px">List comments for a single commitList comments for a single commit.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/commits/{pathv3}/comments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoCommitsShaCodeComments(owner, repo, shaCode, body, callback)</td>
    <td style="padding:15px">Create a commit comment.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/commits/{pathv3}/comments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoCompareBaseIdHeadId(owner, repo, baseId, headId, callback)</td>
    <td style="padding:15px">Compare two commits</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/compare/{pathv3}...{pathv4}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoContentsPath(owner, repo, pathParam, body, callback)</td>
    <td style="padding:15px">Delete a file.
This method deletes a file in a repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/contents/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoContentsPath(owner, repo, pathParam, pathParamQuery, ref, callback)</td>
    <td style="padding:15px">Get contents.
This method returns the contents of a file or directory in a repository.
Files and symlinks support a custom media type for getting the raw content.
Directories and submodules do not support custom media types.
Note: This API supports files up to 1 megabyte in size.
Here can be many outcomes. For details see "http://developer.github.com/v3/repos/contents/"</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/contents/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putReposOwnerRepoContentsPath(owner, repo, pathParam, body, callback)</td>
    <td style="padding:15px">Create a file.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/contents/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoContributors(owner, repo, anon, callback)</td>
    <td style="padding:15px">Get list of contributors.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/contributors?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoDeployments(owner, repo, callback)</td>
    <td style="padding:15px">Users with pull access can view deployments for a repository</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/deployments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoDeployments(owner, repo, body, callback)</td>
    <td style="padding:15px">Users with push access can create a deployment for a given ref</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/deployments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoDeploymentsIdStatuses(owner, repo, id, callback)</td>
    <td style="padding:15px">Users with pull access can view deployment statuses for a deployment</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/deployments/{pathv3}/statuses?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoDeploymentsIdStatuses(owner, repo, id, body, callback)</td>
    <td style="padding:15px">Create a Deployment Status
Users with push access can create deployment statuses for a given deployment:</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/deployments/{pathv3}/statuses?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoDownloads(owner, repo, callback)</td>
    <td style="padding:15px">Deprecated. List downloads for a repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/downloads?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoDownloadsDownloadId(owner, repo, downloadId, callback)</td>
    <td style="padding:15px">Deprecated. Delete a download.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/downloads/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoDownloadsDownloadId(owner, repo, downloadId, callback)</td>
    <td style="padding:15px">Deprecated. Get a single download.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/downloads/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoEvents(owner, repo, callback)</td>
    <td style="padding:15px">Get list of repository events.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/events?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoForks(owner, repo, sort = 'newes', callback)</td>
    <td style="padding:15px">List forks.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/forks?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoForks(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a fork.
Forking a Repository happens asynchronously. Therefore, you may have to wai
a short period before accessing the git objects. If this takes longer than 5
minutes, be sure to contact Support.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/forks?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoGitBlobs(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a Blob.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/blobs?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoGitBlobsShaCode(owner, repo, shaCode, callback)</td>
    <td style="padding:15px">Get a Blob.
Since blobs can be any arbitrary binary data, the input and responses for
the blob API takes an encoding parameter that can be either utf-8 or
base64. If your data cannot be losslessly sent as a UTF-8 string, you can
base64 encode it.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/blobs/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoGitCommits(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a Commit.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/commits?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoGitCommitsShaCode(owner, repo, shaCode, callback)</td>
    <td style="padding:15px">Get a Commit.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/commits/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoGitRefs(owner, repo, callback)</td>
    <td style="padding:15px">Get all References</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/refs?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoGitRefs(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a Reference</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/refs?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoGitRefsRef(owner, repo, ref, callback)</td>
    <td style="padding:15px">Delete a Reference
Example: Deleting a branch: DELETE /repos/octocat/Hello-World/git/refs/heads/feature-a
Example: Deleting a tag:        DELETE /repos/octocat/Hello-World/git/refs/tags/v1.0</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/refs/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoGitMatchingRefs(owner, repo, ref, callback)</td>
    <td style="padding:15px">Get all Matching References</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/matching-refs/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoGitRefsRef(owner, repo, ref, callback)</td>
    <td style="padding:15px">Get a Reference</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/refs/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepoGitRefsRef(owner, repo, ref, body, callback)</td>
    <td style="padding:15px">Update a Reference</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/refs/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoGitTags(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a Tag Object.
Note that creating a tag object does not create the reference that makes a
tag in Git. If you want to create an annotated tag in Git, you have to do
this call to create the tag object, and then create the refs/tags/[tag]
reference. If you want to create a lightweight tag, you only have to create
the tag reference - this call would be unnecessary.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/tags?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoGitTagsShaCode(owner, repo, shaCode, callback)</td>
    <td style="padding:15px">Get a Tag.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/tags/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoGitTrees(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a Tree.
The tree creation API will take nested entries as well. If both a tree and
a nested path modifying that tree are specified, it will overwrite the
contents of that tree with the new path contents and write a new tree out.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/trees?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoGitTreesShaCode(owner, repo, shaCode, recursive, callback)</td>
    <td style="padding:15px">Get a Tree.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/git/trees/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoHooks(owner, repo, callback)</td>
    <td style="padding:15px">Get list of hooks.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/hooks?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoHooks(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a hook.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/hooks?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoHooksHookId(owner, repo, hookId, callback)</td>
    <td style="padding:15px">Delete a hook.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/hooks/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoHooksHookId(owner, repo, hookId, callback)</td>
    <td style="padding:15px">Get single hook.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/hooks/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepoHooksHookId(owner, repo, hookId, body, callback)</td>
    <td style="padding:15px">Edit a hook.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/hooks/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoHooksHookIdTests(owner, repo, hookId, callback)</td>
    <td style="padding:15px">Test a push hook.
This will trigger the hook with the latest push to the current repository
if the hook is subscribed to push events. If the hook is not subscribed
to push events, the server will respond with 204 but no test POST will
be generated.
Note: Previously /repos/:owner/:repo/hooks/:id/tes</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/hooks/{pathv3}/tests?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoIssues(owner, repo, filter = 'assigned', state = 'open', labels, sort = 'created', direction = 'asc', since, callback)</td>
    <td style="padding:15px">List issues for a repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoIssues(owner, repo, body, callback)</td>
    <td style="padding:15px">Create an issue.
Any user with pull access to a repository can create an issue.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoIssuesComments(owner, repo, direction, sort = 'created', since, callback)</td>
    <td style="padding:15px">List comments in a repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/comments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoIssuesCommentsCommentId(owner, repo, commentId, callback)</td>
    <td style="padding:15px">Delete a comment.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/comments/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoIssuesCommentsCommentId(owner, repo, commentId, callback)</td>
    <td style="padding:15px">Get a single comment.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/comments/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepoIssuesCommentsCommentId(owner, repo, commentId, body, callback)</td>
    <td style="padding:15px">Edit a comment.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/comments/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoIssuesEvents(owner, repo, callback)</td>
    <td style="padding:15px">List issue events for a repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/events?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoIssuesEventsEventId(owner, repo, eventId, callback)</td>
    <td style="padding:15px">Get a single event.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/events/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoIssuesNumber(owner, repo, number, callback)</td>
    <td style="padding:15px">Get a single issue</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepoIssuesNumber(owner, repo, number, body, callback)</td>
    <td style="padding:15px">Edit an issue.
Issue owners and users with push access can edit an issue.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoIssuesNumberComments(owner, repo, number, callback)</td>
    <td style="padding:15px">List comments on an issue.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/{pathv3}/comments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoIssuesNumberComments(owner, repo, number, body, callback)</td>
    <td style="padding:15px">Create a comment.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/{pathv3}/comments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoIssuesNumberEvents(owner, repo, number, callback)</td>
    <td style="padding:15px">List events for an issue.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/{pathv3}/events?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoIssuesNumberLabels(owner, repo, number, callback)</td>
    <td style="padding:15px">Remove all labels from an issue.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/{pathv3}/labels?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoIssuesNumberLabels(owner, repo, number, callback)</td>
    <td style="padding:15px">List labels on an issue.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/{pathv3}/labels?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoIssuesNumberLabels(owner, repo, number, body, callback)</td>
    <td style="padding:15px">Add labels to an issue.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/{pathv3}/labels?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putReposOwnerRepoIssuesNumberLabels(owner, repo, number, body, callback)</td>
    <td style="padding:15px">Replace all labels for an issue.
Sending an empty array ([]) will remove all Labels from the Issue.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/{pathv3}/labels?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoIssuesNumberLabelsName(owner, repo, number, name, callback)</td>
    <td style="padding:15px">Remove a label from an issue.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/issues/{pathv3}/labels/{pathv4}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoKeys(owner, repo, callback)</td>
    <td style="padding:15px">Get list of keys.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/keys?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoKeys(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a key.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/keys?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoKeysKeyId(owner, repo, keyId, callback)</td>
    <td style="padding:15px">Delete a key.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/keys/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoKeysKeyId(owner, repo, keyId, callback)</td>
    <td style="padding:15px">Get a key</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/keys/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoLabels(owner, repo, callback)</td>
    <td style="padding:15px">List all labels for this repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/labels?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoLabels(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a label.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/labels?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoLabelsName(owner, repo, name, callback)</td>
    <td style="padding:15px">Delete a label.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/labels/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoLabelsName(owner, repo, name, callback)</td>
    <td style="padding:15px">Get a single label.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/labels/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepoLabelsName(owner, repo, name, body, callback)</td>
    <td style="padding:15px">Update a label.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/labels/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoLanguages(owner, repo, callback)</td>
    <td style="padding:15px">List languages.
List languages for the specified repository. The value on the right of a
language is the number of bytes of code written in that language.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/languages?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoMerges(owner, repo, body, callback)</td>
    <td style="padding:15px">Perform a merge.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/merges?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoMilestones(owner, repo, state = 'open', direction, sort = 'due_date', callback)</td>
    <td style="padding:15px">List milestones for a repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/milestones?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoMilestones(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a milestone.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/milestones?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoMilestonesNumber(owner, repo, number, callback)</td>
    <td style="padding:15px">Delete a milestone.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/milestones/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoMilestonesNumber(owner, repo, number, callback)</td>
    <td style="padding:15px">Get a single milestone.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/milestones/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepoMilestonesNumber(owner, repo, number, body, callback)</td>
    <td style="padding:15px">Update a milestone.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/milestones/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoMilestonesNumberLabels(owner, repo, number, callback)</td>
    <td style="padding:15px">Get labels for every issue in a milestone.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/milestones/{pathv3}/labels?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoNotifications(owner, repo, all, participating, since, callback)</td>
    <td style="padding:15px">List your notifications in a repository
List all notifications for the current user.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/notifications?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putReposOwnerRepoNotifications(owner, repo, body, callback)</td>
    <td style="padding:15px">Mark notifications as read in a repository.
Marking all notifications in a repository as "read" removes them from the
default view on GitHub.com.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/notifications?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoPulls(owner, repo, state = 'open', head, base, callback)</td>
    <td style="padding:15px">List pull requests.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoPulls(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a pull request.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoPullsComments(owner, repo, direction, sort = 'created', since, callback)</td>
    <td style="padding:15px">List comments in a repository.
By default, Review Comments are ordered by ascending ID.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/comments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoPullsCommentsCommentId(owner, repo, commentId, callback)</td>
    <td style="padding:15px">Delete a comment.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/comments/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoPullsCommentsCommentId(owner, repo, commentId, callback)</td>
    <td style="padding:15px">Get a single comment.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/comments/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepoPullsCommentsCommentId(owner, repo, commentId, body, callback)</td>
    <td style="padding:15px">Edit a comment.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/comments/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoPullsNumber(owner, repo, number, callback)</td>
    <td style="padding:15px">Get a single pull request.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepoPullsNumber(owner, repo, number, body, callback)</td>
    <td style="padding:15px">Update a pull request.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoPullsNumberComments(owner, repo, number, callback)</td>
    <td style="padding:15px">List comments on a pull request.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/{pathv3}/comments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoPullsNumberComments(owner, repo, number, body, callback)</td>
    <td style="padding:15px">Create a comment.
  #TODO Alternative input ( http://developer.github.com/v3/pulls/comments/ )
  description: |
    Alternative Input.
    Instead of passing commit_id, path, and position you can reply to an
    existing Pull Request Comment like this:

        body
           Required string
        in_reply_to
           Required number - Comment id to reply to.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/{pathv3}/comments?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoPullsNumberCommits(owner, repo, number, callback)</td>
    <td style="padding:15px">List commits on a pull request.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/{pathv3}/commits?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoPullsNumberFiles(owner, repo, number, callback)</td>
    <td style="padding:15px">List pull requests files.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/{pathv3}/files?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoPullsNumberMerge(owner, repo, number, callback)</td>
    <td style="padding:15px">Get if a pull request has been merged.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/{pathv3}/merge?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putReposOwnerRepoPullsNumberMerge(owner, repo, number, body, callback)</td>
    <td style="padding:15px">Merge a pull request (Merge Button's)</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/pulls/{pathv3}/merge?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoReadme(owner, repo, ref, callback)</td>
    <td style="padding:15px">Get the README.
This method returns the preferred README for a repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/readme?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoReleases(owner, repo, callback)</td>
    <td style="padding:15px">Users with push access to the repository will receive all releases (i.e., published releases and draft releases). Users with pull access will receive published releases only</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/releases?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoReleases(owner, repo, body, callback)</td>
    <td style="padding:15px">Create a release
Users with push access to the repository can create a release.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/releases?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoReleasesAssetsId(owner, repo, id, callback)</td>
    <td style="padding:15px">Delete a release asset</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/releases/assets/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoReleasesAssetsId(owner, repo, id, callback)</td>
    <td style="padding:15px">Get a single release asset</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/releases/assets/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepoReleasesAssetsId(owner, repo, id, body, callback)</td>
    <td style="padding:15px">Edit a release asset
Users with push access to the repository can edit a release asset.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/releases/assets/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoReleasesId(owner, repo, id, callback)</td>
    <td style="padding:15px">Users with push access to the repository can delete a release.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/releases/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoReleasesId(owner, repo, id, callback)</td>
    <td style="padding:15px">Get a single release</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/releases/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchReposOwnerRepoReleasesId(owner, repo, id, body, callback)</td>
    <td style="padding:15px">Users with push access to the repository can edit a release</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/releases/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoReleasesIdAssets(owner, repo, id, callback)</td>
    <td style="padding:15px">List assets for a release</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/releases/{pathv3}/assets?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoStargazers(owner, repo, callback)</td>
    <td style="padding:15px">List Stargazers.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/stargazers?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoStatsCodeFrequency(owner, repo, callback)</td>
    <td style="padding:15px">Get the number of additions and deletions per week.
Returns a weekly aggregate of the number of additions and deletions pushed
to a repository.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/stats/code_frequency?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoStatsCommitActivity(owner, repo, callback)</td>
    <td style="padding:15px">Get the last year of commit activity data.
Returns the last year of commit activity grouped by week. The days array
is a group of commits per day, starting on Sunday.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/stats/commit_activity?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoStatsContributors(owner, repo, callback)</td>
    <td style="padding:15px">Get contributors list with additions, deletions, and commit counts.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/stats/contributors?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoStatsParticipation(owner, repo, callback)</td>
    <td style="padding:15px">Get the weekly commit count for the repo owner and everyone else.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/stats/participation?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoStatsPunchCard(owner, repo, callback)</td>
    <td style="padding:15px">Get the number of commits per hour in each day.
Each array contains the day number, hour number, and number of commits
0-6 Sunday - Saturday
0-23 Hour of day
Number of commits

For example, [2, 14, 25] indicates that there were 25 total commits, during
the 2.00pm hour on Tuesdays. All times are based on the time zone of
individual commits.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/stats/punch_card?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoStatusesRef(owner, repo, ref, callback)</td>
    <td style="padding:15px">List Statuses for a specific Ref.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/statuses/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postReposOwnerRepoStatusesRef(owner, repo, ref, body, callback)</td>
    <td style="padding:15px">Create a Status.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/statuses/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoSubscribers(owner, repo, callback)</td>
    <td style="padding:15px">List watchers.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/subscribers?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteReposOwnerRepoSubscription(owner, repo, callback)</td>
    <td style="padding:15px">Delete a Repository Subscription.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/subscription?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoSubscription(owner, repo, callback)</td>
    <td style="padding:15px">Get a Repository Subscription.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/subscription?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putReposOwnerRepoSubscription(owner, repo, body, callback)</td>
    <td style="padding:15px">Set a Repository Subscription</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/subscription?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoTags(owner, repo, callback)</td>
    <td style="padding:15px">Get list of tags.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/tags?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoTeams(owner, repo, callback)</td>
    <td style="padding:15px">Get list of teams</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/teams?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoWatchers(owner, repo, callback)</td>
    <td style="padding:15px">List Stargazers. New implementation.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/watchers?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getReposOwnerRepoArchiveFormatPath(owner, repo, archiveFormat = 'tarball', pathParam, callback)</td>
    <td style="padding:15px">Get archive link.
This method will return a 302 to a URL to download a tarball or zipball
archive for a repository. Please make sure your HTTP framework is
configured to follow redirects or you will need to use the Location header
to make a second GET request.
Note: For private repositories, these links are temporary and expire quickly.</td>
    <td style="padding:15px">{base_path}/{version}/repos/{pathv1}/{pathv2}/{pathv3}/{pathv4}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getRepositories(since, callback)</td>
    <td style="padding:15px">List all public repositories.
This provides a dump of every public repository, in the order that they
were created.
Note: Pagination is powered exclusively by the since parameter. is the
Link header to get the URL for the next page of repositories.</td>
    <td style="padding:15px">{base_path}/{version}/repositories?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getSearchCodeWithOptions(queryData, callback)</td>
    <td style="padding:15px">Search code.</td>
    <td style="padding:15px">{base_path}/{version}/search/code?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getSearchIssuesWithOptions(queryData, callback)</td>
    <td style="padding:15px">Find issues by state and keyword. (This method returns up to 100 results per page.)</td>
    <td style="padding:15px">{base_path}/{version}/search/issues?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getSearchRepositories(order = 'desc', q, sort = 'stars', callback)</td>
    <td style="padding:15px">Search repositories.</td>
    <td style="padding:15px">{base_path}/{version}/search/repositories?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getSearchUsers(order = 'desc', q, sort = 'followers', callback)</td>
    <td style="padding:15px">Search users.</td>
    <td style="padding:15px">{base_path}/{version}/search/users?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteTeamsTeamId(teamId, callback)</td>
    <td style="padding:15px">Delete team.
In order to delete a team, the authenticated user must be an owner of the
org that the team is associated with.</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getTeamsTeamId(teamId, callback)</td>
    <td style="padding:15px">Get team.</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchTeamsTeamId(teamId, body, callback)</td>
    <td style="padding:15px">Edit team.
In order to edit a team, the authenticated user must be an owner of the org
that the team is associated with.</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getTeamsTeamIdMembers(teamId, callback)</td>
    <td style="padding:15px">List team members.
In order to list members in a team, the authenticated user must be a member
of the team.</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}/members?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteTeamsTeamIdMembersUsername(teamId, username, callback)</td>
    <td style="padding:15px">The "Remove team member" API is deprecated and is scheduled for removal in the next major version of the API. We recommend using the Remove team membership API instead. It allows you to remove both active and pending memberships.

Remove team member.
In order to remove a user from a team, the authenticated user must have 'admin'
permissions to the team or be an owner of the org that the team is associated
with.
NOTE This does not delete the user, it just remove them from the team.</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}/members/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getTeamsTeamIdMembersUsername(teamId, username, callback)</td>
    <td style="padding:15px">The "Get team member" API is deprecated and is scheduled for removal in the next major version of the API. We recommend using the Get team membership API instead. It allows you to get both active and pending memberships.

Get team member.
In order to get if a user is a member of a team, the authenticated user mus
be a member of the team.</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}/members/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putTeamsTeamIdMembersUsername(teamId, username, callback)</td>
    <td style="padding:15px">The API (described below) is deprecated and is scheduled for removal in the next major version of the API. We recommend using the Add team membership API instead. It allows you to invite new organization members to your teams.

Add team member.
In order to add a user to a team, the authenticated user must have 'admin'
permissions to the team or be an owner of the org that the team is associated
with.</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}/members/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteTeamsTeamIdMembershipsUsername(teamId, username, callback)</td>
    <td style="padding:15px">Remove team membership.
In order to remove a membership between a user and a team, the authenticated user must have 'admin' permissions to the team or be an owner of the organization that the team is associated with. NOTE: This does not delete the user, it just removes their membership from the team.</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}/memberships/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getTeamsTeamIdMembershipsUsername(teamId, username, callback)</td>
    <td style="padding:15px">Get team membership.
In order to get a user's membership with a team, the authenticated user must be a member of the team or an owner of the team's organization.</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}/memberships/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putTeamsTeamIdMembershipsUsername(teamId, username, callback)</td>
    <td style="padding:15px">Add team membership.
In order to add a membership between a user and a team, the authenticated user must have 'admin' permissions to the team or be an owner of the organization that the team is associated with.

If the user is already a part of the team's organization (meaning they're on at least one other team in the organization), this endpoint will add the user to the team.

If the user is completely unaffiliated with the team's organization (meaning they're on none of the organization's team...(description truncated)</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}/memberships/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getTeamsTeamIdRepos(teamId, callback)</td>
    <td style="padding:15px">List team repos</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}/repos?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteTeamsTeamIdReposOwnerRepo(teamId, owner, repo, callback)</td>
    <td style="padding:15px">In order to remove a repository from a team, the authenticated user must be an owner of the org that the team is associated with. NOTE: This does not delete the repository, it just removes it from the team.</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}/repos/{pathv2}/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getTeamsTeamIdReposOwnerRepo(teamId, owner, repo, callback)</td>
    <td style="padding:15px">Check if a team manages a repository</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}/repos/{pathv2}/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putTeamsTeamIdReposOwnerRepo(teamId, owner, repo, callback)</td>
    <td style="padding:15px">In order to add a repository to a team, the authenticated user must be an owner of the org that the team is associated with. Also, the repository must be owned by the organization, or a direct fork of a repository owned by the organization.</td>
    <td style="padding:15px">{base_path}/{version}/teams/{pathv1}/repos/{pathv2}/{pathv3}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUser(callback)</td>
    <td style="padding:15px">Get the authenticated user.</td>
    <td style="padding:15px">{base_path}/{version}/user?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">patchUser(body, callback)</td>
    <td style="padding:15px">Update the authenticated user.</td>
    <td style="padding:15px">{base_path}/{version}/user?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteUserEmails(body, callback)</td>
    <td style="padding:15px">Delete email address(es).
You can include a single email address or an array of addresses.</td>
    <td style="padding:15px">{base_path}/{version}/user/emails?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserEmails(callback)</td>
    <td style="padding:15px">List email addresses for a user.
In the final version of the API, this method will return an array of hashes
with extended information for each email address indicating if the address
has been verified and if it's primary email address for GitHub.
Until API v3 is finalized, use the application/vnd.github.v3 media type to
get other response format.</td>
    <td style="padding:15px">{base_path}/{version}/user/emails?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postUserEmails(body, callback)</td>
    <td style="padding:15px">Add email address(es).
You can post a single email address or an array of addresses.</td>
    <td style="padding:15px">{base_path}/{version}/user/emails?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserFollowers(callback)</td>
    <td style="padding:15px">List the authenticated user's followers</td>
    <td style="padding:15px">{base_path}/{version}/user/followers?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserFollowing(callback)</td>
    <td style="padding:15px">List who the authenticated user is following.</td>
    <td style="padding:15px">{base_path}/{version}/user/following?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteUserFollowingUsername(username, callback)</td>
    <td style="padding:15px">Unfollow a user.
Unfollowing a user requires the user to be logged in and authenticated with
basic auth or OAuth with the user:follow scope.</td>
    <td style="padding:15px">{base_path}/{version}/user/following/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserFollowingUsername(username, callback)</td>
    <td style="padding:15px">Check if you are following a user.</td>
    <td style="padding:15px">{base_path}/{version}/user/following/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putUserFollowingUsername(username, callback)</td>
    <td style="padding:15px">Follow a user.
Following a user requires the user to be logged in and authenticated with
basic auth or OAuth with the user:follow scope.</td>
    <td style="padding:15px">{base_path}/{version}/user/following/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserIssues(filter = 'assigned', state = 'open', labels, sort = 'created', direction = 'asc', since, callback)</td>
    <td style="padding:15px">List issues.
List all issues across owned and member repositories for the authenticated
user.</td>
    <td style="padding:15px">{base_path}/{version}/user/issues?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserKeys(callback)</td>
    <td style="padding:15px">List your public keys.
Lists the current user's keys. Management of public keys via the API requires
that you are authenticated through basic auth, or OAuth with the 'user', 'write:public_key' scopes.</td>
    <td style="padding:15px">{base_path}/{version}/user/keys?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postUserKeys(body, callback)</td>
    <td style="padding:15px">Create a public key.</td>
    <td style="padding:15px">{base_path}/{version}/user/keys?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteUserKeysKeyId(keyId, callback)</td>
    <td style="padding:15px">Delete a public key. Removes a public key. Requires that you are authenticated via Basic Auth or via OAuth with at least admin:public_key scope.</td>
    <td style="padding:15px">{base_path}/{version}/user/keys/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserKeysKeyId(keyId, callback)</td>
    <td style="padding:15px">Get a single public key.</td>
    <td style="padding:15px">{base_path}/{version}/user/keys/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserOrgs(callback)</td>
    <td style="padding:15px">List public and private organizations for the authenticated user.</td>
    <td style="padding:15px">{base_path}/{version}/user/orgs?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserRepos(type = 'all', callback)</td>
    <td style="padding:15px">List repositories for the authenticated user. Note that this does not include
repositories owned by organizations which the user can access. You can lis
user organizations and list organization repositories separately.</td>
    <td style="padding:15px">{base_path}/{version}/user/repos?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">postUserRepos(body, callback)</td>
    <td style="padding:15px">Create a new repository for the authenticated user. OAuth users must supply
repo scope.</td>
    <td style="padding:15px">{base_path}/{version}/user/repos?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserStarred(direction, sort = 'created', callback)</td>
    <td style="padding:15px">List repositories being starred by the authenticated user.</td>
    <td style="padding:15px">{base_path}/{version}/user/starred?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteUserStarredOwnerRepo(owner, repo, callback)</td>
    <td style="padding:15px">Unstar a repository</td>
    <td style="padding:15px">{base_path}/{version}/user/starred/{pathv1}/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserStarredOwnerRepo(owner, repo, callback)</td>
    <td style="padding:15px">Check if you are starring a repository.</td>
    <td style="padding:15px">{base_path}/{version}/user/starred/{pathv1}/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putUserStarredOwnerRepo(owner, repo, callback)</td>
    <td style="padding:15px">Star a repository.</td>
    <td style="padding:15px">{base_path}/{version}/user/starred/{pathv1}/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserSubscriptions(callback)</td>
    <td style="padding:15px">List repositories being watched by the authenticated user.</td>
    <td style="padding:15px">{base_path}/{version}/user/subscriptions?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">deleteUserSubscriptionsOwnerRepo(owner, repo, callback)</td>
    <td style="padding:15px">Stop watching a repository</td>
    <td style="padding:15px">{base_path}/{version}/user/subscriptions/{pathv1}/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserSubscriptionsOwnerRepo(owner, repo, callback)</td>
    <td style="padding:15px">Check if you are watching a repository.</td>
    <td style="padding:15px">{base_path}/{version}/user/subscriptions/{pathv1}/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">putUserSubscriptionsOwnerRepo(owner, repo, callback)</td>
    <td style="padding:15px">Watch a repository.</td>
    <td style="padding:15px">{base_path}/{version}/user/subscriptions/{pathv1}/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUserTeams(callback)</td>
    <td style="padding:15px">List all of the teams across all of the organizations to which the authenticated user belongs. This method requires user or repo scope when authenticating via OAuth.</td>
    <td style="padding:15px">{base_path}/{version}/user/teams?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsers(since, callback)</td>
    <td style="padding:15px">Get all users.
This provides a dump of every user, in the order that they signed up for GitHub.
Note: Pagination is powered exclusively by the since parameter. Use the Link
header to get the URL for the next page of users.</td>
    <td style="padding:15px">{base_path}/{version}/users?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsername(username, callback)</td>
    <td style="padding:15px">Get a single user.</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameEvents(username, callback)</td>
    <td style="padding:15px">If you are authenticated as the given user, you will see your private events. Otherwise, you'll only see public events.</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/events?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameEventsOrgsOrg(username, org, callback)</td>
    <td style="padding:15px">This is the user's organization dashboard. You must be authenticated as the user to view this.</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/events/orgs/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameFollowers(username, callback)</td>
    <td style="padding:15px">List a user's followers</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/followers?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameFollowingTargetUser(username, targetUser, callback)</td>
    <td style="padding:15px">Check if one user follows another.</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/following/{pathv2}?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameGists(username, since, callback)</td>
    <td style="padding:15px">List a users gists.</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/gists?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameKeys(username, callback)</td>
    <td style="padding:15px">List public keys for a user.
Lists the verified public keys for a user. This is accessible by anyone.</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/keys?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameOrgs(username, callback)</td>
    <td style="padding:15px">List all public organizations for a user.</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/orgs?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameReceivedEvents(username, callback)</td>
    <td style="padding:15px">These are events that you'll only see public events.</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/received_events?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameReceivedEventsPublic(username, callback)</td>
    <td style="padding:15px">List public events that a user has received</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/received_events/public?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameRepos(username, type = 'all', callback)</td>
    <td style="padding:15px">List public repositories for the specified user.</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/repos?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameStarred(username, callback)</td>
    <td style="padding:15px">List repositories being starred by a user.</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/starred?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
  <tr>
    <td style="padding:15px">getUsersUsernameSubscriptions(username, callback)</td>
    <td style="padding:15px">List repositories being watched by a user.</td>
    <td style="padding:15px">{base_path}/{version}/users/{pathv1}/subscriptions?{query}</td>
    <td style="padding:15px">Yes</td>
  </tr>
</table>
<br>

### Authentication

This document will go through the steps for authenticating the GitHub System adapter with the authentication methods we have worked with in the past. Properly configuring the properties for an adapter in IAP is critical for getting the adapter online. You can read more about adapter authentication <a href="https://docs.itential.com/opensource/docs/authentication" target="_blank">HERE</a>. 

#### Basic Authentication
The GitHub System adapter requires Basic Authentication. If you change authentication methods, you should change this section accordingly and merge it back into the adapter repository.

STEPS  
1. Ensure you have access to a GitHub System server and that it is running
2. Follow the steps in the README.md to import the adapter into IAP if you have not already done so
3. Use the properties below for the ```properties.authentication``` field
```json
"authentication": {
  "auth_method": "basic user_password",
  "username": "<username>",
  "password": "<access_token>",
  "auth_field": "header.headers.Authorization",
  "auth_field_format": "Basic {b64}{username}:{password}{/b64}",
  "auth_logging": false
}
```
4. Restart the adapter. If your properties were set correctly, the adapter should go online. 

#### Static Token Authentication
The GitHub System adapter authenticates with a static token.

STEPS  
1. Ensure you have access to a GitHub System server and that it is running
2. Follow the steps in the README.md to import the adapter into IAP if you have not already done so
3. Use the properties below for the ```properties.authentication``` field
```json
"authentication": {
  "auth_method": "static_token",
  "token": "<access_token>",
  "auth_field": "header.headers.Authorization",
  "auth_field_format": "token {token}",
  "auth_logging": false
}
```
you can leave all of the other properties in the authentication section, they will not be used when the auth_method is static_token.
4. Restart the adapter. If your properties were set correctly, the adapter should go online. 


#### Troubleshooting
- Make sure you copied over the correct username and password.
- Turn on debug level logs for the adapter in IAP Admin Essentials.
- Turn on auth_logging for the adapter in IAP Admin Essentials (adapter properties).
- Investigate the logs - in particular:
  - The FULL REQUEST log to make sure the proper headers are being sent with the request.
  - The FULL BODY log to make sure the payload is accurate.
  - The CALL RETURN log to see what the other system is telling us.
- Credentials should be ** masked ** by the adapter so make sure you verify the username and password - including that there are erroneous spaces at the front or end.
- Remember when you are done to turn auth_logging off as you do not want to log credentials.

## Additional Information

### Enhancements

#### Adding a Second Instance of an Adapter

You can add a second instance of this adapter without adding new code on the file system. To do this go into the IAP Admin Essentials and add a new service config for this adapter. The two instances of the adapter should have unique ids. In addition, they should point to different instances (unique host and port) of the other system.

#### Adding Adapter Calls

There are multiple ways to add calls to an existing adapter.

The easiest way would be to use the Adapter Builder update process. This process takes in a Swagger or OpenAPI document, allows you to select the calls you want to add and then generates a zip file that can be used to update the adapter. Once you have the zip file simply put it in the adapter directory and execute `npm run adapter:update`.

```bash
mv updatePackage.zip adapter-github
cd adapter-github
npm run adapter:update
```

If you do not have a Swagger or OpenAPI document, you can use a Postman Collection and convert that to an OpenAPI document using APIMatic and then follow the first process.

If you want to manually update the adapter that can also be done the key thing is to make sure you update all of the right files. Within the entities directory you will find 1 or more entities. You can create a new entity or add to an existing entity. Each entity has an action.json file, any new call will need to be put in the action.json file. It will also need to be added to the enum for the ph_request_type in the appropriate schema files. Once this configuration is complete you will need to add the call to the adapter.js file and, in order to make it available as a workflow task in IAP, it should also be added to the pronghorn.json file. You can optionally add it to the unit and integration test files. There is more information on how to work on each of these files in the <a href="https://docs.itential.com/opensource/docs/adapters" target="_blank">Adapter Technical Resources</a> on our Documentation Site.

```text
Files to update
* entities/<entity>/action.json: add an action
* entities/<entity>/schema.json (or the schema defined on the action): add action to the enum for ph_request_type
* adapter.js: add the new method and make sure it calls the proper entity and action
* pronghorn.json: add the new method
* test/unit/adapterTestUnit.js (optional but best practice): add unit test(s) - function is there, any required parameters error when not passed in
* test/integration/adapterTestIntegration.js (optional but best practice): add integration test
```

#### Adding Adapter Properties

While changing adapter properties is done in the service instance configuration section of IAP, adding properties has to be done in the adapter. To add a property you should edit the propertiesSchema.json with the proper information for the property. In addition, you should modify the sampleProperties to have the new property in it.

```text
Files to update
* propertiesSchema.json: add the new property and how it is defined
* sampleProperties: add the new property with a default value
* test/unit/adapterTestUnit.js (optional but best practice): add the property to the global properties
* test/integration/adapterTestIntegration.js (optional but best practice): add the property to the global properties
```

#### Changing Adapter Authentication

Often an adapter is built before knowing the authentication and authentication processes can also change over time. The adapter supports many different kinds of authentication but it does require configuration. Some forms of authentication can be defined entirely with the adapter properties but others require configuration.

```text
Files to update
* entities/.system/action.json: change the getToken action as needed
* entities/.system/schemaTokenReq.json: add input parameters (external name is name in other system)
* entities/.system/schemaTokenResp.json: add response parameters (external name is name in other system)
* propertiesSchema.json: add any new property and how it is defined
* sampleProperties: add any new property with a default value
* test/unit/adapterTestUnit.js (optional but best practice): add the property to the global properties
* test/integration/adapterTestIntegration.js (optional but best practice): add the property to the global properties
```

#### Enhancing Adapter Integration Tests

The adapter integration tests are written to be able to test in either stub (standalone) mode or integrated to the other system. However, if integrating to the other system, you may need to provide better data than what the adapter provides by default as that data is likely to fail for create and update. To provide better data, edit the adapter integration test file. Make sure you do not remove the marker and keep custom code below the marker so you do not impact future migrations. Once the edits are complete, run the integration test as it instructs you to above. When you run integrated to the other system, you can also save mockdata for future use by changing the isSaveMockData flag to true.

```text
Files to update
* test/integration/adapterTestIntegration.js: add better data for the create and update calls so that they will not fail.
```

As mentioned previously, for most of these changes as well as other possible changes, there is more information on how to work on an adapter in the <a href="https://docs.itential.com/opensource/docs/adapters" target="_blank">Adapter Technical Resources</a> on our Documentation Site.

### Contributing

First off, thanks for taking the time to contribute!

The following is a set of rules for contributing.

#### Code of Conduct

This project and everyone participating in it is governed by the Code of Conduct. By participating, you are expected to uphold this code. Please report unacceptable behavior to support@itential.com.

#### How to Contribute

Follow the contributing guide (here)[https://gitlab.com/itentialopensource/adapters/contributing-guide]

### Helpful Links

<a href="https://docs.itential.com/opensource/docs/adapters" target="_blank">Adapter Technical Resources</a>

### Node Scripts

There are several node scripts that now accompany the adapter. These scripts are provided to make several activities easier. Many of these scripts can have issues with different versions of IAP as they have dependencies on IAP and Mongo. If you have issues with the scripts please report them to the Itential Adapter Team. Each of these scripts are described below.

<table border="1" class="bordered-table">
  <tr>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Run</span></th>
    <th bgcolor="lightgrey" style="padding:15px"><span style="font-size:12.0pt">Description</span></th>
  </tr>
  <tr>
    <td style="padding:15px">npm run adapter:install</td>
    <td style="padding:15px">Provides an easier way to install the adapter.</td>
  </tr>
  <tr>
    <td style="padding:15px">npm run adapter:checkMigrate</td>
    <td style="padding:15px">Checks whether your adapter can and should be migrated to the latest foundation.</td>
  </tr>
  <tr>
    <td style="padding:15px">npm run adapter:findPath</td>
    <td style="padding:15px">Can be used to see if the adapter supports a particular API call.</td>
  </tr>
  <tr>
    <td style="padding:15px">npm run adapter:migrate</td>
    <td style="padding:15px">Provides an easier way to update your adapter after you download the migration zip from Itential DevSite.</td>
  </tr>
  <tr>
    <td style="padding:15px">npm run adapter:update</td>
    <td style="padding:15px">Provides an easier way to update your adapter after you download the update zip from Itential DevSite.</td>
  </tr>
  <tr>
    <td style="padding:15px">npm run adapter:revert</td>
    <td style="padding:15px">Allows you to revert after a migration or update if it resulted in issues.</td>
  </tr>
  <tr>
    <td style="padding:15px">npm run troubleshoot</td>
    <td style="padding:15px">Provides a way to troubleshoot the adapter - runs connectivity, healthcheck and basic get.</td>
  </tr>
  <tr>
    <td style="padding:15px">npm run connectivity</td>
    <td style="padding:15px">Provides a connectivity check to the Servicenow system.</td>
  </tr>
  <tr>
    <td style="padding:15px">npm run healthcheck</td>
    <td style="padding:15px">Checks whether the configured healthcheck call works to Servicenow.</td>
  </tr>
  <tr>
    <td style="padding:15px">npm run basicget</td>
    <td style="padding:15px">Checks whether the basic get calls works to Servicenow.</td>
  </tr>
</table>
<br>

## Troubleshoot

Run `npm run troubleshoot` to start the interactive troubleshooting process. The command allows you to verify and update connection, authentication as well as healthcheck configuration. After that it will test these properties by sending HTTP request to the endpoint. If the tests pass, it will persist these changes into IAP.

You also have the option to run individual commands to perform specific test:

- `npm run healthcheck` will perform a healthcheck request of with current setting.
- `npm run basicget` will perform some non-parameter GET request with current setting.
- `npm run connectivity` will perform networking diagnostics of the adatper endpoint.

### Connectivity Issues

1. You can run the adapter troubleshooting script which will check connectivity, run the healthcheck and run basic get calls.

```bash
npm run troubleshoot
```

2. Verify the adapter properties are set up correctly.

```text
Go into the Itential Platform GUI and verify/update the properties
```

3. Verify there is connectivity between the Itential Platform Server and GitHub Server.

```text
ping the ip address of GitHub server
try telnet to the ip address port of GitHub
execute a curl command to the other system
```

4. Verify the credentials provided for GitHub.

```text
login to GitHub using the provided credentials
```

5. Verify the API of the call utilized for GitHub Healthcheck.

```text
Go into the Itential Platform GUI and verify/update the properties
```

### Functional Issues

Adapter logs are located in `/var/log/pronghorn`. In older releases of the Itential Platform, there is a `pronghorn.log` file which contains logs for all of the Itential Platform. In newer versions, adapters can be configured to log into their own files.


